#!/usr/local/bin/perl
# search.cgi
# Search for manual pages, and display a list of matches or an exact page

require './man-lib.pl';
use Config;
&ReadParse();
$in{'for'} || &error($text{'search_efor'});
&header($text{'search_title'}, "");

@for = split(/\s+/, $in{'for'});
@howto = split(/\s+/, $config{'howto_dir'});
@doc = split(/\s+/, $config{'doc_dir'});
map { $section{$_}++ } split(/\0/, $in{'section'});

if ($section{'doc'}) {
	# Look in the system documentation directory (usually /usr/doc)
	foreach $d (@doc) {
		push(@rv, map { [ $text{'search_doc'},
				   "view_doc.cgi?file=".&urlize($_->[0]),
				   substr($_->[0], length($d)+1),
				   $_->[1], $_->[3] ? 1 : .1 ] }
			     &find_contents($d, \@for,
					    $howto[0], 1, $in{'exact'}));
		}
	}
if ($section{'howto'}) {
	# Look in the HOWTO directory
	foreach $h (@howto) {
		push(@rv, map { [ $text{'search_howto'},
			      "view_howto.cgi?file=".&urlize($_->[0]),
			      $_->[2], $_->[1],
			      $_->[3] ? 2 : .2 ] }
		     &find_contents($h, \@for, undef, 0, $in{'exact'}));
		}
	@rv = grep { $_->[2] !~ /^index/i } @rv;
	}
if ($section{'kernel'}) {
	# Look in the linux kernel Documentation directory
	 push(@rv, map { [ $text{'search_kernel'},
			   "view_kernel.cgi?file=".&urlize($_->[0]),
			   substr($_->[0], length($config{'kernel_dir'})+1),
			   $_->[1], $_->[3] ? 1 :.1 ] }
		     &find_contents($config{'kernel_dir'}, \@for,
				    undef, 1, $in{'exact'}));
	}
if ($section{'kde'}) {
	# Look in the KDE documentation directory
	 push(@rv, map { [ $text{'search_kde'},
			   "view_kde.cgi?file=".&urlize($_->[0]),
			   substr($_->[0], length($config{'kde_dir'})+1),
			   $_->[1], $_->[3] ? 1 : .1 ] }
		     &find_contents($config{'kde_dir'}, \@for,
				    undef, 1, $in{'exact'}));
	}
if ($section{'perl'}) {
	if ($in{'exact'}) {
		# Check for an exact module name match
		chop($out = `$perl_doc -l $for[0] 2>/dev/null`);
		if ($out) {
			local $doc = &parse_perl_module($out);
			$doc->{'name'} =~ s/^\s*(\S+)\s+-\s+//;
			push(@rv, [ $text{'search_perl'},
				    "view_perl.cgi?mod=$for[0]",
				    $for[0], $doc->{'name'}, 1 ]);
			}
		}
	else {
		# Search the text of all perl modules
		foreach $d ($Config{'sitelib'}, $Config{'privlib'}) {
			open(FIND, "find $d -name '*.pm' -print |");
			PATH: while($path = <FIND>) {
				chop($path);
				local $doc = &parse_perl_module($path);
				foreach $f (@for) {
					next PATH if ($doc->{'name'} !~ /$f/i &&
					        $doc->{'description'} !~ /$f/i);
					}
				$doc->{'name'} =~ s/^\s*(\S+)\s+-\s+//;
				local $modfile =
				   `$perl_doc -l $doc->{'package'} 2>/dev/null`;
				local $exact = 1;
				foreach $f (@for) {
					$exact = .1
					    if ($doc->{'package'} !~ /$f/i);
					}
				if ($doc->{'package'} && $modfile) {
					push(@rv, [ $text{'search_perl'},
					  "view_perl.cgi?mod=$doc->{'package'}",
					  $doc->{'package'}, $doc->{'name'},
					  $exact ]);
					}
				}
			close(FIND);
			}
		}
	}
if ($section{'help'}) {
	# Look in the webmin module help pages
	opendir(DIR, "..");
	foreach $m (readdir(DIR)) {
		# Is this a module with help
		local $dir = "../$m/help";
		next if (!-d $dir || $m =~ /^\./ || -l "../$m");
		local %minfo = &get_module_info($m);
		next if (!%minfo || !&check_os_support(\%minfo));

		# Check the help pages
		local @pfx;
		opendir(DIR, $dir);
		while($f = readdir(DIR)) {
			push(@pfx, $1) if ($f =~ /^([^\.]+)\.html$/);
			}
		closedir(DIR);
		HELP: foreach $p (&unique(@pfx)) {
			local $file = &help_file($m, $p);
			open(HELP, $file);
			local @st = stat($file);
			read(HELP, $help, $st[7]);
			close(HELP);
			if ($help =~ /<header>([^<]+)<\/header>/) {
				$header = $1;
				}
			else { next; }
			$help =~ s/<include\s+(\S+)>/inchelp($1)/ge;
			$help =~ s/<[^>]+>//g;
			local $exact = 1;
			foreach $f (@for) {
				next HELP if ($help !~ /$f/i);
				$exact = 0 if ($header !~ /$f/i);
				}
			if (!$in{'exact'} || $exact) {
				push(@rv, [ $text{'search_help'},
					    "/help.cgi/$m/$p?x=1",
					    "$m/$p", $header,
					    $exact ? 2 : .2 ]);
				}
			}
		}
	}
if ($section{'man'}) {
	# Look in manual pages
	$cmd = $config{'search_cmd'};
	$cmd =~ s/PAGE/"$for[0]"/;
	open(MAN, "$cmd |");
	LINE: while(<MAN>) {
		if (/(([^,\s]+).*)\s*\((\S+)\)\s+-\s+(.*)/ &&
		    !$done{$2,$3,$4}++) {
			local ($page, $sect, $desc) = ($1, $3, $4);
			local @pp = split(/[\s+,]/, $page);
			local $ex = 1;
			foreach $f (@for) {
				$ex = 0 if (&indexof($f, @pp) < 0);
				}
			if ($in{'exact'}) {
				# All keywords must be in page names
				next if (!$ex);
				}
			else {
				# Keywords must be page name or desc
				foreach $f (@for) {
					next LINE if ($desc !~ /$f/i &&
						      &indexof($f, @pp) < 0);
					}
				}
			push(@rv, [ $text{'search_man'},
				    "view_man.cgi?page=$pp[0]&sec=$3",
				    "$pp[0] ($sect)", $desc,
				    $ex ? 3 : .3 ]);
			}
		}
	close(MAN);
	}

if (@rv == 1) {
	# redirect to the exact page
	&redirect($in{'exact'} ? $rv[0]->[1]
			       : "$rv[0]->[1]&for=".&urlize($in{'for'}));
	exit;
	}

# Display search results
print "<center><font size=+1>",&text('search_for', "<tt>$in{'for'}</tt>"),
      "</font></center>\n";
print "<hr>\n";
if (@rv) {
	@rv = sort { $b->[4] <=> $a->[4] } @rv;
	print "<table border width=100%>\n";
	print "<tr $tb> <td><b>$text{'search_file'}</b></td> ",
	      "<td><b>$text{'search_type'}</b></td> ",
	      "<td><b>$text{'search_desc'}</b></td> </tr>\n";
	foreach $r (@rv) {
		print "<tr $cb>\n";
		print "<td><a href='$r->[1]&for=".&urlize($in{'for'})."'>",
		      "$r->[2]</a></td>\n";
		print "<td nowrap>$r->[0]</td>\n";
		print "<td>",$r->[3] ? $r->[3] : "<br>","</td>\n";
		print "</tr>\n";
		}
	print "</table><p>\n";
	}
else {
	print "<p><b>",&text('search_none', "<tt>$in{'for'}</tt>"),"</b><p>\n";
	}

print "<hr>\n";
&footer("", $text{'index_return'});

# find_contents(directory, &strings, [exclude], [descend], [nameonly])
# Find some string in a directory of files
sub find_contents
{
opendir(DIR, $_[0]);
local @f = readdir(DIR);
closedir(DIR);
local @rv;
foreach $f (@f) {
	next if ($f =~ /^\./);
	local $p = "$_[0]/$f";
	next if ($p eq $_[2]);
	if (-d $p) {
		# go into subdirectory
		push(@rv, &find_contents($p, $_[1], $_[2], $_[3], $_[4]))
			if ($_[3]);
		}
	else {
		# Skip non-text or HTML files
		local $ff = $f;
		$ff =~ s/\.gz$//i;
		next if ($ff !~ /\.(txt|htm|html|doc)$/ &&
			 $ff =~ /\.[A-Za-z0-9]+$/);

		local $matches = 0;
		foreach $s (@{$_[1]}) {
			$matches++ if ($p =~ /$s/i);
			}
		if ($_[4]) {
			# just compare filename
			if ($matches == @{$_[1]}) {
				local ($desc, $data) = &read_doc_file($p);
				if ($desc !~ /^#!/) {
					push(@rv, [ $p, $desc, $f, $matches ]);
					}
				}
			}
		else {
			# compare file contents
			local ($desc, $data) = &read_doc_file($p);
			local $all = 1;
			foreach $s (@{$_[1]}) {
				$all = 0 if ($data !~ /$s/i);
				}
			if ($all && $first !~ /^#!/) {
				push(@rv, [ $p, $desc, $f, $matches ]);
				}
			}
		}
	}
return @rv;
}

# read_doc_file(filename)
# Returns desc, data
sub read_doc_file
{
local ($two, $first, $title, $data);
open(FILE, $_[0]);
read(FILE, $two, 2);
if ($two eq "\037\213") {
	close(FILE);
	open(FILE, "gunzip -c $p |");
	}
seek(FILE, 0, 0);
while(<FILE>) {
	$data .= $_;
	if (/[A-Za-z0-9]/ && !/\$\S+:/ && !$first) {
		chop($first = $_);
		}
	}
close(FILE);
if ($data =~ /<\s*title\s*>([\000-\177]{0,200})<\s*\/\s*title\s*>/i) {
	$title = $1;
	}
return ($title ? $title : $first =~ /<.*>/ ? undef : $first, $data);
}

# parse_perl_module(file)
sub parse_perl_module
{
local (%doc, $inside);
open(MOD, $_[0]);
while(<MOD>) {
	if (/^\s*package\s+(\S+)\s*;/ && !$doc{'package'}) {
		$doc{'package'} = $1;
		}
	elsif (/^=head1\s+(\S+)/i) {
		$inside = $1;
		}
	elsif (/^=cut/i) {
		undef($inside);
		}
	elsif ($inside) {
		$doc{lc($inside)} .= $_;
		}
	}
close(MOD);
return \%doc;
}

# help_file(dir, prefix)
sub help_file
{
local $lang = "$_[0]/$_[1].$current_lang.html";
local $def = "$_[0]/$_[1].html";
return -r $lang ? $lang : $def;
}

# inchelp(path)
sub inchelp
{
local $inc;
local $ipath = &help_file($dir, $_[0]);
open(INC, $ipath) || return "<i>".&text('search_einclude', $_[0])."</i><br>\n";
local @st = stat(INC);
read(INC, $inc, $st[7]);
close(INC);
return $inc;
}


