#!/usr/local/bin/perl
# batch_exec.cgi
# Execute create/modify/delete commands in a batch file

require './user-lib.pl';
&ReadParseMime();
if ($in{'file'}) {
	$data = $in{'file'};
	}
elsif ($in{'local'}) {
	open(LOCAL, $in{'local'}) || &error($text{'batch_elocal'});
	while(<LOCAL>) {
		$data .= $_;
		}
	close(LOCAL);
	}
else {
	&error($text{'batch_efile'});
	}

&header($text{'batch_title'}, "");
print "<hr>\n";
$| = 1;

# Work out a good base UID for new users
&lock_user_files();
&my_setpwent();
while(@tmp = &my_getpwent()) {
	$used{$tmp[2]}++;
	$taken{$tmp[0]}++;
	}
&my_endpwent();
$newuid = int($config{'base_uid'});

# Work out a good base GID for new groups
&my_setgrent();
while(@tmp = &my_getgrent()) {
	$gused{$tmp[2]}++;
	$gtaken{$tmp[0]}++;
	}
&my_endgrent();
$newgid = int($config{'base_gid'});

# Process the file
print "<pre>\n";
$lnum = 0;
foreach $line (split(/[\r\n]+/, $data)) {
	$lnum++;
	$line =~ s/#.*$//;
	next if ($line !~ /\S/);
	local @line = split(/:/, $line, -1);
	local %user;
	if ($line[0] eq 'create') {
		# Creating a new user
		if (&passfiles_type() == 2) {
			# SYSV-style passwd and shadow information
			if (@line != 13) {
				print &text('batch_elen', $lnum, 13),"\n";
				next;
				}
			$user{'min'} = $line[8];
			$user{'max'} = $line[9];
			$user{'warn'} = $line[10];
			$user{'inactive'} = $line[11];
			$user{'expire'} = $line[12];
			$user{'change'} = int(time() / (60*60*24));
			}
		elsif (&passfiles_type() == 1) {
			# BSD master.passwd information
			if (@line != 11) {
				print &text('batch_elen', $lnum, 11),"\n";
				next;
				}
			$user{'class'} = $line[8];
			$user{'change'} = $line[9];
			$user{'expire'} = $line[10];
			}
		else {
			# Classic passwd file information
			if (@line != 8) {
				print &text('batch_elen', $lnum, 8),"\n";
				next;
				}
			}

		# Parse common fields
		if (!$line[1]) {
			print &text('batch_eline', $lnum),"\n";
			next;
			}
		$user{'user'} = $line[1];
		if ($taken{$user{'user'}}) {
			print &text('batch_euser', $lnum, $user{'user'}),"\n";
			next;
			}
		if ($line[3] !~ /^\d+$/) {
			# make up a UID
			while($used{$newuid}) {
				$newuid++;
				}
			$user{'uid'} = $newuid;
			}
		else {
			# use the given UID!!
			$user{'uid'} = $line[3];
			}
		$used{$user{'uid'}}++;
		if ($line[6] !~ /^\//) {
			print &text('batch_ehome', $lnum, $line[6]),"\n";
			next;
			}
		$user{'home'} = $line[6];
		if (!-r $line[7]) {
			print &text('batch_eshell', $lnum, $line[7]),"\n";
			next;
			}
		$user{'shell'} = $line[7];
		$user{'real'} = $line[5];
		if ($line[4] !~ /^\d+$/) {
			# Need to create a new group for the user
			if ($gtaken{$user{'user'}}) {
				print &text('batch_egtaken', $lnum, $user{'user'}),"\n";
				next;
				}
			while($gused{$newgid}) {
				$newgid++;
				}
			local %group;
			$group{'group'} = $user{'user'};
			$user{'gid'} = $group{'gid'} = $newgid;
			&create_group(\%group);
			}
		else {
			$user{'gid'} = $line[4];
			}

		# Create the user!
		if ($in{'makehome'} && !-d $user{'home'}) {
			if (!mkdir($user{'home'}, oct($config{'homedir_perms'}))) {
				print &text('batch_emkdir', $user{'home'}, $!),"\n";
				}
			chmod(oct($config{'homedir_perms'}), $user{'home'});
			chown($user{'uid'}, $user{'gid'}, $user{'home'});
			}
		if ($line[2] eq 'x') {
			# No login allowed
			$user{'pass'} = $config{'lock_string'};
			$user{'passmode'} = 1;
			}
		elsif ($line[2] eq '') {
			# No password needed
			$user{'pass'} = '';
			$user{'passmode'} = 0;
			}
		else {
			# Normal password
			$salt = chr(int(rand(26))+65) . chr(int(rand(26))+65);
			$user{'pass'} = crypt($line[2], $salt);
			$user{'passmode'} = 3;
			$user{'plainpass'} = $line[2];
			}
		&create_user(\%user);
		&other_modules("useradmin_create_user", \%user);

		if ($in{'copy'} && $in{'makehome'}) {
			# Copy files to user's home directory
			local $uf = $config{'user_files'};
			if ($group = &my_getgrgid($user{'gid'})) {
				$uf =~ s/\$group/$group/g;
				}
			$uf =~ s/\$gid/$user{'gid'}/g;
			&copy_skel_files($uf, $user{'home'},
					 $user{'uid'}, $user{'gid'});
			}

		print "<b>",&text('batch_created',$user{'user'}),"</b>\n";
		}
	elsif ($line[0] eq 'delete') {
		# Deleting an existing user
		if (@line != 2) {
			print &text('batch_elen', $lnum, 2),"\n";
			next;
			}
		local @ulist = &list_users();
		local ($user) = grep { $_->{'user'} eq $line[1] } @ulist;
		if (!$user) {
			print &text('batch_enouser', $lnum, $line[1]),"\n";
			next;
			}
		%user = %$user;

		# Delete the user entry
		&other_modules("useradmin_delete_user", \%user);
		$ENV{'USERADMIN_USER'} = $user{'user'};
		$ENV{'USERADMIN_ACTION'} = 'DELETE_USER';
		&making_changes();
		&delete_user(\%user);
		&made_changes();

		# Delete his home directory
		if ($in{'delhome'} && $user{'home'} !~ /^\/+$/) {
			if ($config{'delete_only'}) {
				&lock_file($user{'home'});
				&system_logged("find \"$user{'home'}\" ! -type d -user $user{'uid'} | xargs rm -f >/dev/null 2>&1");
				&system_logged("find \"$user{'home'}\" -type d -user $user{'uid'} | xargs rmdir >/dev/null 2>&1");
				rmdir($user{'home'});
				&unlock_file($user{'home'});
				}
			else {
				&system_logged("rm -rf \"$user{'home'}\" >/dev/null 2>&1");
				}
			}

		print "<b>",&text('batch_deleted',$user{'user'}),"</b>\n";
		}
	elsif ($line[0] eq 'modify') {
		# Modifying an existing user
		if (&passfiles_type() == 2) {
			# SYSV-style passwd and shadow information
			if (@line != 14) {
				print &text('batch_elen', $lnum, 14),"\n";
				next;
				}
			}
		elsif (&passfiles_type() == 1) {
			# BSD master.passwd information
			if (@line != 12) {
				print &text('batch_elen', $lnum, 12),"\n";
				next;
				}
			}
		else {
			# Classic passwd file information
			if (@line != 9) {
				print &text('batch_elen', $lnum, 9),"\n";
				next;
				}
			}
		local @ulist = &list_users();
		local ($user) = grep { $_->{'user'} eq $line[1] } @ulist;
		if (!$user) {
			print &text('batch_enouser', $lnum, $line[1]),"\n";
			next;
			}
		%olduser = %user = %$user;
		$user{'olduser'} = $user->{'user'};

		# Update supplied fields
		$user{'user'} = $line[2] if ($line[2] ne '');
		if ($line[3] eq 'x') {
			# No login allowed
			$user{'pass'} = $config{'lock_string'};
			$user{'passmode'} = 1;
			}
		elsif ($line[3] ne '') {
			# Normal password
			$salt = chr(int(rand(26))+65) . chr(int(rand(26))+65);
			$user{'pass'} = crypt($line[3], $salt);
			$user{'passmode'} = 3;
			$user{'plainpass'} = $line[3];
			}
		else {
			# No change
			$user{'passmode'} = 4;
			}
		$user{'uid'} = $line[4] if ($line[4] ne '');
		$user{'gid'} = $line[5] if ($line[5] ne '');
		$user{'real'} = $line[6] if ($line[6] ne '');
		$user{'home'} = $line[7] if ($line[7] ne '');
		$user{'shell'} = $line[8] if ($line[8] ne '');
		if (&passfiles_type() == 2) {
			# SYSV-style passwd and shadow information
			$user{'min'} = $line[9] if ($line[9] ne '');
			$user{'max'} = $line[10] if ($line[10] ne '');
			$user{'warn'} = $line[11] if ($line[11] ne '');
			$user{'inactive'} = $line[12] if ($line[12] ne '');
			$user{'expire'} = $line[13] if ($line[13] ne '');
			$user{'change'} = int(time() / (60*60*24))
				if ($line[3] ne '');
			}
		elsif (&passfiles_type() == 1) {
			# BSD master.passwd information
			$user{'class'} = $line[9] if ($line[9] ne '');
			$user{'change'} = $line[10] if ($line[10] ne '');
			$user{'expire'} = $line[11] if ($line[11] ne '');
			}

		# Move home directory if needed
		if ($olduser{'home'} ne $user{'home'} && $in{'movehome'} &&
		    $user{'home'} ne '/' && $olduser{'home'} ne '/') {
			if (-d $olduser{'home'} && !-e $user{'home'}) {
				local $out = &backquote_logged(
					"mv \"$olduser{'home'}\" ".
					"\"$user{'home'}\" 2>&1");
				if ($?) { &error(&text('batch_emove',
						 $lnum, $out)); }
				}
			}

		# Change UIDs and GIDs?
		if ($olduser{'gid'} != $user{'gid'} && $in{'chgid'}) {
			if ($in{'chgid'} == 1) {
				&recursive_change($user{'home'},$olduser{'uid'},
					  $olduser{'gid'}, -1, $user{'gid'});
				}
			else {
				&recursive_change("/", $olduser{'uid'},
					  $olduser{'gid'}, -1, $user{'gid'});
				}
			}
		if ($olduser{'uid'} != $user{'uid'} && $in{'chuid'}) {
			if ($in{'chuid'} == 1) {
				&recursive_change($user{'home'},$olduser{'uid'},
						  -1, $user{'uid'}, -1);
				}
			else {
				&recursive_change("/", $olduser{'uid'},
						  -1, $user{'uid'}, -1);
				}
			}

		# Actually modify the user
		$ENV{'USERADMIN_USER'} = $user{'user'};
		$ENV{'USERADMIN_ACTION'} = 'MODIFY_USER';
		&making_changes();
		&modify_user(\%olduser, \%user);
		&made_changes();
		&other_modules("useradmin_modify_user", \%user);

		print "<b>",&text('batch_modified',$olduser{'user'}),"</b>\n";
		}
	else {
		print &text('batch_eaction', $lnum, $line[0]),"\n";
		next;
		}
	}
print "</pre>\n";
&unlock_user_files();

# XXXX logging?

print "<hr>\n";
&footer("", $text{'index_return'});

